#-----------------------------------------------------------------------------
#
# Notification Framework: "Observer" Design Pattern for python
# (c) 2001, 2002, 2003 Sebastien Bigaret
# 
# This file is part of the Notification Framework.
# 
# The Notification Framework is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License as published
# by the Free Software Foundation; either version 2 of the License, or (at
# your option) any later version.
# 
# The Notification Framework is distributed in the hope that it will be
# useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
# 
# You should have received a copy of the GNU General Public License along
# with the Notification Framework; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
# 
#-----------------------------------------------------------------------------

"""
Logging methods.

  All methods log(), warn() and trace() are disabled by default. To enable
  them set the environment variable 'NOTIFICATION_FRAMEWORK_TRACE' (for
  debugging purpose mainly)

CVS Information

  $Id: log.py,v 1.2 2006/01/09 07:01:10 ariza Exp $
"""

__version__='$Revision: 1.2 $'[11:-2]

import os

if os.environ.get('NOTIFICATION_FRAMEWORK_TRACE', None):
  INFO='INFO'
  WARNING='WARNING'
  TRACE='TRACE'
  import sys
  from time import strftime
  def LOG(subsystem, severity, summary, detail, error=None, reraise=None):
    sys.stderr.write('------\n%s LOG[%s]: %s\n%s\n'%\
                     (strftime('%Y-%m-%dT%H:%M:%S'),severity,summary,detail))
      
  def log(summary, detail, error=None, reraise=None):
    LOG(subsystem='NotificationFramework', severity=INFO,
        summary=summary, detail=detail+'\n', error=error, reraise=reraise)
  def warn(summary, detail, error=None, reraise=None):
    LOG(subsystem='NotificationFramework', severity=WARNING,
        summary=summary, detail=detail+'\n', error=error, reraise=reraise)
  def trace(summary, detail, error=None, reraise=None):
    LOG(subsystem='NotificationFramework', severity=INFO,
        summary=summary, detail=detail+'\n', error=error, reraise=reraise)
else:
  def log(summary, detail, error=None, reraise=None):
    pass
  def warn(summary, detail, error=None, reraise=None):
    pass
  def trace(summary, detail, error=None, reraise=None):
    pass
